/*
 * main.c - get time sample application
 *
 * Copyright (C) 2014 Texas Instruments Incorporated - http://www.ti.com/
 *
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * Application Name     -   Get time
 * Application Overview -   This sample application demonstrates how
 *                          to connect to an SNTP server and request for time
 *                          information. The application processes the received
 *                          data and displays the time on console
 * Application Details  -   http://processors.wiki.ti.com/index.php/CC31xx_Get_Time_Application
 *                          doc\examples\get_time.pdf
 ******************************************************************************/
#include "CC3100/include/simplelink.h"
#include "CC3100/source/protocol.h"
#include "CC3100/sl_common.h"
#include <stdio.h>

/****DEFINES*******************************************************************/
#define APPLICATION_VERSION "1.2.0"

#define SL_STOP_TIMEOUT 0xFF

#define MAX_BUF_SIZE    48

#define TIME2013        3565987200       /* 43 years + 11 days of leap years */
#define YEAR2013        2013
#define SEC_IN_MIN      60
#define SEC_IN_HOUR     3600
#define SEC_IN_DAY      86400

//Values for below macros shall be modified for setting the time-zone
#define GMT_TIME_ZONE_HR    -7
#define GMT_TIME_ZONE_MIN   00

#define	SW0_IO PORTGbits.RG15

/****Application specific status/error codes***********************************/
typedef enum{
    DEVICE_NOT_IN_STATION_MODE = -0x7D0,        /* Choosing this number to avoid overlap w/ host-driver's error codes */
    SNTP_SEND_ERROR = DEVICE_NOT_IN_STATION_MODE - 1,
    SNTP_RECV_ERROR = SNTP_SEND_ERROR - 1,
    SNTP_SERVER_RESPONSE_ERROR = SNTP_RECV_ERROR - 1,
    STATUS_CODE_MAX = -0xBB8
}e_AppStatusCodes;

/****GLOBAL VARIABLES**********************************************************/
UINT32  g_Status = 0;
UINT8 Buff[110];

struct{
    UINT32 DestinationIP;
    UINT32 elapsedSec;
    UINT32 uGeneralVar;
    UINT32 uGeneralVar1;
    UINT16 ccLen;
    INT32  SockID;
    INT32  sGeneralVar;
    UINT8  time[30];
    UINT8 *ccPtr;
}appData;

/****CONTSTANT*****************************************************************/
/*******************************************************************************
 * List of SNTP servers
 *          hostname         |        IP        |       location
 * nist1-nj2.ustiming.org    | 165.193.126.229  | Weehawken, NJ
 * nist1-pa.ustiming.org     | 206.246.122.250  | Hatfield, PA
 * time-a.nist.gov           | 129.6.15.28      | NIST, Gaithersburg, Maryland
 * time-b.nist.gov           | 129.6.15.29      | NIST, Gaithersburg, Maryland
 * time-c.nist.gov           | 129.6.15.30      | NIST, Gaithersburg, Maryland
 * ntp-nist.ldsbc.edu        | 198.60.73.8      | LDSBC, Salt Lake City, Utah
 * nist1-macon.macon.ga.us   | 98.175.203.200   | Macon, Georgia
 * For more SNTP server link visit 'http://tf.nist.gov/tf-cgi/servers.cgi'
 ******************************************************************************/
const UINT8 SNTPserver[30] = "time-c.nist.gov"; /* Add any one of the above servers */

/* Tuesday is the 1st day in 2013 - the relative year */
const UINT8 daysOfWeek2013[7][3] = {{"Tue"},
                                   {"Wed"},
                                   {"Thu"},
                                   {"Fri"},
                                   {"Sat"},
                                   {"Sun"},
                                   {"Mon"}};

const UINT8 monthOfYear[12][3] = {{"Jan"},
                                 {"Feb"},
                                 {"Mar"},
                                 {"Apr"},
                                 {"May"},
                                 {"Jun"},
                                 {"Jul"},
                                 {"Aug"},
                                 {"Sep"},
                                 {"Oct"},
                                 {"Nov"},
                                 {"Dec"}};

const UINT8 numOfDaysPerMonth[12] = {31,28,31,30,31,30,31,31,30,31,30,31};

const UINT8 digits[] = "0123456789";

/****STATIC FUNCTION DEFINITIONS***********************************************/

static INT32 disconnectFromAP();

static INT32 initializeAppVariables();
static void  displayBanner();

static INT32 getHostIP();
static INT32 createConnection();
static INT32 getSNTPTime(INT16 gmt_hr, INT16 gmt_min);
static UINT16 my_itoa(INT16 cNum, UINT8 *cString);

/***Application's entry point**************************************************/
int main(int argc, char** argv)
{
    INT32 retVal = -1;

    DDPCONbits.JTAGEN = 0;          // Disable Jtag

    SYSTEMConfig(80000000ul, SYS_CFG_WAIT_STATES | SYS_CFG_PCACHE);

    INTEnableSystemMultiVectoredInt();

    //initialize the system-clock of the MCU
    initClk();

    //Configure command line interface
    CLI_Configure((80000000ul/(1 << OSCCONbits.PBDIV)), 921600ul);
    displayBanner();

    while(SW0_IO);

    retVal = configureSimpleLinkToDefaultState();
    if(retVal < 0)
    {
        if (DEVICE_NOT_IN_STATION_MODE == retVal)
            CLI_Write("Failed to configure the device in its default state\r\n");

        LOOP_FOREVER();
    }
    retVal = initializeAppVariables();
    ASSERT_ON_ERROR(retVal);
    CLI_Write("Device is configured in default state\r\n");

    retVal = sl_Start(0, 0);
    if ((retVal < 0) ||
        (ROLE_STA != retVal) )
    {
        CLI_Write("Failed to start the device\r\n");
        LOOP_FOREVER();
    }

    CLI_Write("Device started as STATION\r\n");

    //Connecting to WLAN AP
    retVal = establishConnectionWithAP();
    if(retVal < 0)
    {
        CLI_Write("Failed to establish connection with an AP\r\n");
        LOOP_FOREVER();
    }

    CLI_Write("Connection established with AP and IP is acquired\r\n");

    retVal = getHostIP();
    if(retVal < 0)
    {
        CLI_Write("Unable to get host IP\r\n");
        LOOP_FOREVER();
    }
    appData.SockID = createConnection();
    if(appData.SockID < 0)
        LOOP_FOREVER();

    retVal = getSNTPTime(GMT_TIME_ZONE_HR,GMT_TIME_ZONE_MIN);
    if(retVal < 0)
        LOOP_FOREVER();

    retVal = sl_Close(appData.SockID);
    if(retVal < 0)
        LOOP_FOREVER();

    retVal = disconnectFromAP();
    if(retVal < 0)
    {
        CLI_Write("Failed to disconnect from the AP\r\n");
        LOOP_FOREVER();
    }

    return 0;
}


/****ASYNCHRONOUS EVENT HANDLERS***********************************************/

/*******************************************************************************
 * brief This function handles WLAN events
 * param[in] pWlanEvents is the event passed to the handler
 * return none
 ******************************************************************************/
void SimpleLinkWlanEventHandler(SlWlanEvent_t *pWlanEvent)
{
    if(pWlanEvent == NULL)
    {
        CLI_Write("[WLAN EVENT] NULL Pointer Error\r\n");
        return;
    }

    switch(pWlanEvent->Event)
    {
        case SL_WLAN_CONNECT_EVENT:
        {
            SET_STATUS_BIT(g_Status, STATUS_BIT_CONNECTION);

            /*
             * Information about the connected AP (like name, MAC etc) will be
             * available in 'slWlanConnectAsyncResponse_t' - Applications
             * can use it if required
             *
             * slWlanConnectAsyncResponse_t *pEventData = NULL;
             * pEventData = &pWlanEvent->EventData.STAandP2PModeWlanConnected;
             *
             */
        }
        break;

        case SL_WLAN_DISCONNECT_EVENT:
        {
            slWlanConnectAsyncResponse_t*  pEventData = NULL;

            CLR_STATUS_BIT(g_Status, STATUS_BIT_CONNECTION);
            CLR_STATUS_BIT(g_Status, STATUS_BIT_IP_ACQUIRED);

            pEventData = &pWlanEvent->EventData.STAandP2PModeDisconnected;

            /* If the user has initiated 'Disconnect' request, 'reason_code' is SL_USER_INITIATED_DISCONNECTION */
            if(SL_USER_INITIATED_DISCONNECTION == pEventData->reason_code)
            {
                CLI_Write("Device disconnected from the AP on application's request\r\n");
            }
            else
            {
                CLI_Write("Device disconnected from the AP on an ERROR!\r\n");
            }
        }
        break;

        default:
        {
            CLI_Write("[WLAN EVENT] Unexpected event\r\n");
        }
        break;
    }
}

/*******************************************************************************
 * brief This function handles events for IP address acquisition via DHCP
 *       indication
 * param[in] pNetAppEvent is the event passed to the handler
 * return None
 ******************************************************************************/
void SimpleLinkNetAppEventHandler(SlNetAppEvent_t *pNetAppEvent)
{
    if(pNetAppEvent == NULL)
    {
        CLI_Write("[NETAPP EVENT] NULL Pointer Error\r\n");
        return;
    }

    switch(pNetAppEvent->Event)
    {
        case SL_NETAPP_IPV4_IPACQUIRED_EVENT:
        {
            SET_STATUS_BIT(g_Status, STATUS_BIT_IP_ACQUIRED);

            /*
             * Information about the connected AP's IP, gateway, DNS etc
             * will be available in 'SlIpV4AcquiredAsync_t' - Applications
             * can use it if required
             *
             * SlIpV4AcquiredAsync_t *pEventData = NULL;
             * pEventData = &pNetAppEvent->EventData.ipAcquiredV4;
             * <gateway_ip> = pEventData->gateway;
             *
             */
        }
        break;

        default:
        {
            CLI_Write("[NETAPP EVENT] Unexpected event\r\n");
        }
        break;
    }
}

/*******************************************************************************
 * brief This function handles callback for the HTTP server events
 * param[in] pHttpEvent - Contains the relevant event information
 * param[in] pHttpResponse - Should be filled by the user with the
 *           relevant response information
 * return None
 ******************************************************************************/
void SimpleLinkHttpServerCallback(SlHttpServerEvent_t *pHttpEvent,
                                  SlHttpServerResponse_t *pHttpResponse)
{
    /*
     * This application doesn't work with HTTP server - Hence these
     * events are not handled here
     */
    CLI_Write("[HTTP EVENT] Unexpected event\r\n");
}

/*******************************************************************************
 * brief This function handles general error events indication
 * param[in] pDevEvent is the event passed to the handler
 * return None
 ******************************************************************************/
void SimpleLinkGeneralEventHandler(SlDeviceEvent_t *pDevEvent)
{
    /*
     * Most of the general errors are not FATAL are are to be handled
     * appropriately by the application
     */
    CLI_Write("[GENERAL EVENT]\r\n");
}

/*******************************************************************************
 * brief This function handles socket events indication
 * param[in] pSock is the event passed to the handler
 * return None
 ******************************************************************************/
void SimpleLinkSockEventHandler(SlSockEvent_t *pSock)
{
    if(pSock == NULL)
    {
        CLI_Write("[SOCK EVENT] NULL Pointer Error\r\n");
        return;
    }

    switch( pSock->Event )
    {
        case SL_SOCKET_TX_FAILED_EVENT:
        {
            /*
            * TX Failed
            *
            * Information about the socket descriptor and status will be
            * available in 'SlSockEventData_t' - Applications can use it if
            * required
            *
            * SlSockEventData_u *pEventData = NULL;
            * pEventData = & pSock->socketAsyncEvent;
            */
            switch( pSock->socketAsyncEvent.SockTxFailData.status )
            {
                case SL_ECLOSE:
                    CLI_Write("[SOCK EVENT] Close socket operation failed to transmit all queued packets\r\n");
                break;


                default:
                    CLI_Write("[SOCK EVENT] Unexpected event\r\n");
                break;
            }
        }
        break;

        default:
            CLI_Write("[SOCK EVENT] Unexpected event\r\n");
        break;
    }
}

/****Applications**************************************************************/

/*******************************************************************************
 * brief Convert integer to ASCII in decimal base
 * param[in] cNum - integer number to convert
 * param[OUT] cString - output string
 * return number of ASCII characters
 ******************************************************************************/
static _u16 my_itoa(INT16 cNum, UINT8 *cString)
{
    UINT16 length = 0;
    UINT8* ptr = NULL;
    INT16 uTemp = cNum;

    /* value 0 is a special case */
    if (cNum == 0)
    {
        length = 1;
        *cString = '0';

        return length;
    }

    /* Find out the length of the number, in decimal base */
    length = 0;
    while (uTemp > 0)
    {
        uTemp /= 10;
        length++;
    }

    /* Do the actual formatting, right to left */
    uTemp = cNum;
    ptr = cString + length;
    while (uTemp > 0)
    {
        --ptr;
        *ptr = digits[uTemp % 10];
        uTemp /= 10;
    }

    return length;
}

/*******************************************************************************
 * brief Get the required data from the server.
 * param[in] gmt_hr - GMT offset hours
 * param[in] gmt_min - GMT offset minutes
 * return 0 on success, -ve otherwise
 ******************************************************************************/
static INT32 getSNTPTime(INT16 gmt_hr, INT16 gmt_min)
{
    char sec, min, hour, wday, mday, mon;
    int year;
    /*
                                NTP Packet Header:


           0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9  0  1
          +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
          |LI | VN  |Mode |    Stratum    |     Poll      |   Precision    |
          +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
          |                          Root  Delay                           |
          +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
          |                       Root  Dispersion                         |
          +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
          |                     Reference Identifier                       |
          +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
          |                                                                |
          |                    Reference Time-stamp (64)                    |
          |                                                                |
          +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
          |                                                                |
          |                    Originate Time-stamp (64)                    |
          |                                                                |
          +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
          |                                                                |
          |                     Receive Time-stamp (64)                     |
          |                                                                |
          +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
          |                                                                |
          |                     Transmit Time-stamp (64)                    |
          |                                                                |
          +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
          |                 Key Identifier (optional) (32)                 |
          +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
          |                                                                |
          |                                                                |
          |                 Message Digest (optional) (128)                |
          |                                                                |
          |                                                                |
          +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
    */

    SlSockAddrIn_t  LocalAddr;
    SlSockAddr_t Addr;

    UINT8 dataBuf[MAX_BUF_SIZE];
    INT32 retVal = -1;
    INT16 AddrSize = 0;

    /* For time zone with negative GMT value, change minutes to negative for
     * computation */
    if(gmt_hr < 0 && gmt_min > 0)
        gmt_min = gmt_min * (-1);

    memset(dataBuf, 0, sizeof(dataBuf));
    dataBuf[0] = '\x1b';

    Addr.sa_family = AF_INET;
    /* the source port */
    Addr.sa_data[0] = 0x00;
    Addr.sa_data[1] = 0x7B;    /* 123 */
    Addr.sa_data[2] = (_u8)((appData.DestinationIP >> 24) & 0xff);
    Addr.sa_data[3] = (_u8)((appData.DestinationIP >> 16) & 0xff);
    Addr.sa_data[4] = (_u8)((appData.DestinationIP >> 8) & 0xff);
    Addr.sa_data[5] = (_u8) (appData.DestinationIP & 0xff);

    retVal = sl_SendTo(appData.SockID, dataBuf, sizeof(dataBuf), 0,
                     &Addr, sizeof(Addr));
    if (retVal != sizeof(dataBuf))
    {
        /* could not send SNTP request */
        CLI_Write("Device couldn't send SNTP request\r\n");
        ASSERT_ON_ERROR(SNTP_SEND_ERROR);
    }

    AddrSize = sizeof(SlSockAddrIn_t);
    LocalAddr.sin_family = SL_AF_INET;
    LocalAddr.sin_port = 0;
    LocalAddr.sin_addr.s_addr = 0;

    retVal = sl_Bind(appData.SockID,(SlSockAddr_t *)&LocalAddr, AddrSize);
    if(retVal < 0)
        ASSERT_ON_ERROR(retVal);

    retVal = sl_RecvFrom(appData.SockID, dataBuf, sizeof(dataBuf), 0,
                       (SlSockAddr_t *)&LocalAddr,  (SlSocklen_t*)&AddrSize);
    if (retVal <= 0)
    {
        CLI_Write("Device couldn't receive time information\r\n");
        ASSERT_ON_ERROR(SNTP_RECV_ERROR);
    }

    if ((dataBuf[0] & 0x7) != 4)    /* expect only server response */
    {
        /* MODE is not server, abort */
        CLI_Write("Device is expecting response from server only!\r\n");
        ASSERT_ON_ERROR(SNTP_SERVER_RESPONSE_ERROR);
    }
    else
    {
        UINT8 index;

        appData.elapsedSec = dataBuf[40];
        appData.elapsedSec <<= 8;
        appData.elapsedSec += dataBuf[41];
        appData.elapsedSec <<= 8;
        appData.elapsedSec += dataBuf[42];
        appData.elapsedSec <<= 8;
        appData.elapsedSec += dataBuf[43];

        appData.elapsedSec -= TIME2013;

        /* correct the time zone */
        appData.elapsedSec += (gmt_hr * SEC_IN_HOUR);
        appData.elapsedSec += (gmt_min * SEC_IN_MIN);

        appData.ccPtr = &appData.time[0];

        /* day */
        appData.sGeneralVar = appData.elapsedSec/SEC_IN_DAY;
        wday = appData.sGeneralVar % 7;
        memcpy(appData.ccPtr, daysOfWeek2013[wday], 3);
        appData.ccPtr += 3;
        *appData.ccPtr++ = '\x20';

        /* month */
        appData.sGeneralVar %= 365;
        for (index = 0; index < 12; index++)
        {
            appData.sGeneralVar -= numOfDaysPerMonth[index];
            if (appData.sGeneralVar < 0)
                break;
        }

        mon = index;
        memcpy(appData.ccPtr, monthOfYear[index], 3);
        appData.ccPtr += 3;
        *appData.ccPtr++ = '\x20';

        /* date */
        /* restore the day in current month*/
        appData.sGeneralVar += numOfDaysPerMonth[index];
        mday = appData.sGeneralVar + 1;
        appData.ccLen = my_itoa(mday, appData.ccPtr);
        appData.ccPtr += appData.ccLen;
        *appData.ccPtr++ = '\x20';

        /* year */
        /* number of days since beginning of 2013 */
        appData.uGeneralVar = appData.elapsedSec/SEC_IN_DAY;
        appData.uGeneralVar /= 365;
        year = YEAR2013 + appData.uGeneralVar;
        appData.ccLen = my_itoa(year , appData.ccPtr);
        appData.ccPtr += appData.ccLen;
        *appData.ccPtr++ = '\x20';

        /* time */
        appData.uGeneralVar = appData.elapsedSec%SEC_IN_DAY;

        /* number of seconds per hour */
        appData.uGeneralVar1 = appData.uGeneralVar%SEC_IN_HOUR;
        appData.uGeneralVar /= SEC_IN_HOUR;               /* number of hours */
        hour = appData.uGeneralVar;
        appData.ccLen = my_itoa(appData.uGeneralVar, appData.ccPtr);
        appData.ccPtr += appData.ccLen;
        *appData.ccPtr++ = ':';

        /* number of minutes per hour */
        appData.uGeneralVar = appData.uGeneralVar1/SEC_IN_MIN;
        min = appData.uGeneralVar;
        /* number of seconds per minute */
        appData.uGeneralVar1 %= SEC_IN_MIN;
        sec = appData.uGeneralVar1;
        appData.ccLen = my_itoa(appData.uGeneralVar, appData.ccPtr);
        appData.ccPtr += appData.ccLen;
        *appData.ccPtr++ = ':';
        appData.ccLen = my_itoa(appData.uGeneralVar1, appData.ccPtr);
        appData.ccPtr += appData.ccLen;
        *appData.ccPtr++ = '\x20';

        *appData.ccPtr++ = '\0';

        CLI_Write("Server \"");
        CLI_Write((UINT8*)SNTPserver);
        CLI_Write("\" has responded with time information\r\n");
        CLI_Write(appData.time);
        CLI_Write("\r\n");

        //rtccSet2127(sec, min, hour, wday, mday, mon, year);
    }

    return SUCCESS;
}

/*******************************************************************************
 * brief Create UDP socket to communicate with server.
 * param[in] none
 * return Socket descriptor for success otherwise negative
 ******************************************************************************/
static INT32 createConnection()
{
    INT32 sd = 0;

    sd = sl_Socket(SL_AF_INET, SL_SOCK_DGRAM, IPPROTO_UDP);
    if( sd < 0 )
    {
        CLI_Write("Error creating socket\r\n");
    }

    return sd;
}

/*******************************************************************************
 * brief Gets the Server IP address
 * param[in] none
 * return zero for success and -1 for error
 ******************************************************************************/
static INT32 getHostIP()
{
    INT32 status = 0;
    appData.DestinationIP = 0;

    status = sl_NetAppDnsGetHostByName((_i8*)SNTPserver, strlen(SNTPserver),
                                       &appData.DestinationIP, SL_AF_INET);
    ASSERT_ON_ERROR(status);

    return SUCCESS;
}

/*******************************************************************************
 * brief This function disconnects from the connected AP
 * param[in] None
 * return none
 * warning If the WLAN disconnection fails, we will be stuck in this function forever.
 ******************************************************************************/
static INT32 disconnectFromAP()
{
    INT32 retVal = -1;

    /*
     * The function returns 0 if 'Disconnected done', negative number if already disconnected
     * Wait for 'disconnection' event if 0 is returned, Ignore other return-codes
     */
    retVal = sl_WlanDisconnect();
    if(0 == retVal)
    {
        /* Wait */
        while(IS_CONNECTED(g_Status)) { _SlNonOsMainLoopTask(); }
    }

    return SUCCESS;
}

/*******************************************************************************
 * brief This function initializes the application variables
 * param[in] None
 * return 0 on success, negative error-code on error
 ******************************************************************************/
static INT32 initializeAppVariables()
{
    g_Status = 0;
    memset(&appData, 0, sizeof(appData));

    return SUCCESS;
}

/*******************************************************************************
 * brief This function displays the application's banner
 * param None
 * return None
 ******************************************************************************/
static void displayBanner()
{
    CLI_Write(" Get time application - Version ");
    CLI_Write(APPLICATION_VERSION);
    CLI_Write("**************************************************\r\n");
}
